from django.contrib import messages
from django.contrib.messages.storage.base import Message
from django.core import mail
from django.http import HttpResponseRedirect, HttpResponsePermanentRedirect


class StatusCodeAssertionsMixin(object):
    redirect_codes = [
        HttpResponseRedirect.status_code,
        HttpResponsePermanentRedirect.status_code
    ]

    def assert_status_equal(self, response, status_code_or_response):
        status_code = self._get_status_code(status_code_or_response)
        self.assertEqual(
            response.status_code,
            status_code,
            'Response status code is {0}, expected {1}'.format(
                response.status_code,
                status_code,
            )
        )

    def assert_status_in(self, response, status_codes_or_responses):
        status_codes = list(map(self._get_status_code, status_codes_or_responses))
        self.assertIn(
            response.status_code,
            status_codes,
            'Response status code is {0}, expected one of: {1}'.format(
                response.status_code,
                ', '.join(str(code) for code in status_codes),
            )
        )

    def _get_redirect_assertion_message(self, response):
        return 'Response should redirect, but status code is {0}'.format(
            response.status_code
        )

    def assert_redirect(self, response, expected_url=None):
        """
        assertRedirects from Django TestCase follows the redirects chains,
        this assertion does not - which is more like real unit testing
        """
        self.assertIn(
            response.status_code,
            self.redirect_codes,
            self._get_redirect_assertion_message(response),
        )
        if expected_url:
            location_header = response._headers.get('location', None)
            self.assertEqual(
                location_header,
                ('Location', str(expected_url)),
                'Response should redirect to {0}, but it redirects to {1} instead'.format(
                    expected_url,
                    location_header[1],
                )
            )

    def assert_not_redirect(self, response):
        self.assertNotIn(
            response.status_code,
            self.redirect_codes,
            self._get_redirect_assertion_message(response)
        )

    def _get_status_code(self, status_code_or_response):
        try:
            return status_code_or_response.status_code
        except AttributeError:
            return status_code_or_response


class EmailAssertionsMixin(object):
    def assert_emails_in_mailbox(self, count):
        self.assertEqual(
            len(mail.outbox),
            count,
            'There is {0} e-mails in mailbox, expected {1}'.format(
                len(mail.outbox),
                count,
            )
        )

    def _is_email_matching_criteria(self, email, **kwargs):
        for key, value in kwargs.items():
            if getattr(email, key) != value:
                return False
        return True

    def assert_email(self, email, **kwargs):
        for key, value in kwargs.items():
            self.assertEqual(
                getattr(email, key),
                value,
                'Email does not match criteria, expected {0} to be {1} but it is {2}'.format(
                    key,
                    value,
                    getattr(email, key),
                )
            )

    def assert_email_exists(self, **kwargs):
        for email in mail.outbox:
            if self._is_email_matching_criteria(email, **kwargs):
                return
        raise AssertionError('Email matching criteria was not sent')


class MessagesAssertionsMixin(object):
    def assert_messages_sent(self, request, count):
        sent = len(messages.get_messages(request))
        self.assertEqual(
            sent,
            count,
            'There was {0} messages sent, expected {1}.'.format(
                sent,
                count,
            )
        )

    def assert_message_exists(self, request, level, message):
        self.assertIn(
            Message(level=level, message=message),
            messages.get_messages(request),
            'Message matching criteria does not exist'
        )


class _InstanceContext(object):
    """
    Context manager returned by assert_instance_created/deleted.
    """
    def __init__(self, enter_assertion, exit_assertion, model_class, **kwargs):
        self.enter_assertion = enter_assertion
        self.exit_assertion = exit_assertion
        self.model_class = model_class
        self.kwargs = kwargs

    def __enter__(self):
        self.enter_assertion(self.model_class, **self.kwargs)
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.exit_assertion(self.model_class, **self.kwargs)
        return True


class InstanceAssertionsMixin(object):
    """
    ORM-related assertions for testing instance creation and deletion.
    """
    def assert_instance_exists(self, model_class, **kwargs):
        try:
            obj = model_class._default_manager.get(**kwargs)
            self.assertIsNotNone(obj)
        except model_class.DoesNotExist:
            raise AssertionError('No {0} found matching the criteria.'.format(
                    model_class.__name__,
                )
            )

    def assert_instance_does_not_exist(self, model_class, **kwargs):
        try:
            instance = model_class._default_manager.get(**kwargs)
            raise AssertionError('A {0} was found matching the criteria. ({1})'.format(
                model_class.__name__,
                instance,
            ))
        except model_class.DoesNotExist:
            pass

    def assert_instance_created(self, model_class, **kwargs):
        """
        Checks if a model instance was created in the database.

        For example::

        >>> with self.assert_instance_created(Article, slug='lorem-ipsum'):
        ...     Article.objects.create(slug='lorem-ipsum')
        """
        return _InstanceContext(
            self.assert_instance_does_not_exist,
            self.assert_instance_exists,
            model_class,
            **kwargs
        )

    def assert_instance_deleted(self, model_class, **kwargs):
        """
        Checks if the model instance was deleted from the database.

        For example::

        >>> with self.assert_instance_deleted(Article, slug='lorem-ipsum'):
        ...     Article.objects.get(slug='lorem-ipsum').delete()
        """
        return _InstanceContext(
            self.assert_instance_exists,
            self.assert_instance_does_not_exist,
            model_class,
            **kwargs
        )


class CompleteAssertionsMixin(
    StatusCodeAssertionsMixin,
    EmailAssertionsMixin,
    MessagesAssertionsMixin,
    InstanceAssertionsMixin,
):
    pass



import django
from rest_framework import test
from djet import testcases


class APIRequestFactory(testcases.RequestFactory, test.APIRequestFactory):

    def _request(self, method, **kwargs):
        request = super(APIRequestFactory, self)._request(method, **kwargs)
        user = kwargs.get('user')
        token = kwargs.get('token')
        test.force_authenticate(request, user, token)
        return request


class APIViewTransactionTestCase(testcases.ViewTransactionTestCase):
    factory_class = APIRequestFactory


class APIViewTestCase(testcases.ViewTestCase):
    factory_class = APIRequestFactory


if django.VERSION >= (1, 4):
    class APIViewLiveServerTestCase(testcases.ViewLiveServerTestCase):
        factory_class = APIRequestFactory

if django.VERSION >= (1, 5):
    class APIViewSimpleTestCase(testcases.ViewSimpleTestCase):
        factory_class = APIRequestFactory



import os
import datetime
from django.core.files.storage import Storage, default_storage
from django.core.files.uploadedfile import InMemoryUploadedFile
from django.utils import six


class InMemoryStorage(Storage):

    def __init__(self):
        self.files = {}

    def _open(self, name, mode):
        return self.files[name]

    def _save(self, name, content):
        self.files[name] = content
        return name

    def delete(self, name):
        del self.files[name]

    def exists(self, name):
        return name in self.files

    def listdir(self, path):
        directories, files = set(), []
        for name in self.files:
            if name.startswith(path):
                without_path = name[len(path):]
                try:
                    slash_index = without_path.index('/')
                    directories.add(without_path[:slash_index])
                except ValueError:
                    files.append(without_path)
        return list(directories), files

    def size(self, name):
        file_instance = self.files[name]
        file_instance.seek(0, 2)
        return file_instance.tell()

    def url(self, name):
        return name

    def accessed_time(self, name):
        return datetime.datetime.now()

    def created_time(self, name):
        return datetime.datetime.now()

    def modified_time(self, name):
        return datetime.datetime.now()

    def clear(self):
        self.files = {}


class InMemoryStorageMixin(object):
    storage = InMemoryStorage

    def _pre_setup(self):
        self._wrapped_storage = default_storage._wrapped
        default_storage._wrapped = self.storage()
        super(InMemoryStorageMixin, self)._pre_setup()

    def _post_teardown(self):
        super(InMemoryStorageMixin, self)._post_teardown()
        try:
            default_storage.clear()
        except AttributeError:
            pass
        default_storage._wrapped = self._wrapped_storage


def create_inmemory_file(file_name='tmp.txt', content=b'', content_type=None):
    stream = six.BytesIO()
    stream.write(content)
    file = InMemoryUploadedFile(stream, None, file_name, content_type, stream.tell(), None)
    file.seek(0)
    return file


def create_inmemory_image(file_name='tmp.png', format=None, width=200, height=200, content_type=None):
    from PIL import Image
    if not format:
        _, extension = os.path.splitext(file_name)
        format = extension[1:].upper()
    if not content_type:
        content_type = 'image/{0}'.format(format)
    stream = six.BytesIO()
    size = (width, height)
    color = (255, 0, 0, 0)
    image = Image.new('RGBA', size, color)
    image.save(stream, format=format)
    image_file = InMemoryUploadedFile(stream, None, file_name, content_type, stream.tell(), None)
    image_file.seek(0)
    return image_file



import codecs
from setuptools import setup

setup(
    name='djet',
    version='0.2.1',
    description='Set of helpers for easy testing of Django apps.',
    long_description=codecs.open('README.rst', encoding='utf-8').read(),
    license='MIT',
    author='SUNSCRAPERS',
    author_email='info@sunscrapers.com',
    packages=['djet'],
    url='https://github.com/sunscrapers/djet',
    install_requires=[],
    include_package_data=True,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Testing'
    ]
)



from functools import partial
import django
from django import test as django_test
from django.template.response import TemplateResponse


class RequestFactory(django_test.RequestFactory):

    def __init__(self, middleware_classes=None, **defaults):
        super(RequestFactory, self).__init__(**defaults)
        self.middleware_classes = middleware_classes or []
        self._override_shortcuts()

    def _override_shortcuts(self):
        for method in ('get', 'post', 'head', 'delete', 'options', 'put'):
            shortcut = partial(self._request, method)
            setattr(self, method, shortcut)

    def _request(self, method, user=None, path='', **kwargs):
        super_method = getattr(super(RequestFactory, self), method.lower())
        request = super_method(path=path, **kwargs)
        request.user = user
        return request


class MiddlewareType:
    PROCESS_REQUEST = 'process_request'
    PROCESS_VIEW = 'process_view'
    PROCESS_EXCEPTION = 'process_exception'
    PROCESS_RESPONSE = 'process_response'
    PROCESS_TEMPLATE_RESPONSE = 'process_template_response'


class ViewTestCaseMixin(object):
    view_class = None
    view_function = None
    view_kwargs = None
    factory_class = RequestFactory
    middleware_classes = None
    middleware = None

    def _pre_setup(self, *args, **kwargs):
        super(ViewTestCaseMixin, self)._pre_setup(*args, **kwargs)
        if self.factory_class:
            self.factory = self.factory_class(self.get_middleware_classes())

    def get_view_kwargs(self):
        return self.view_kwargs or {}

    def get_middleware_classes(self):
        return self.middleware_classes or []

    def create_view_object(self, request=None, *args, **kwargs):
        view_object = self.view_class(**self.get_view_kwargs())
        view_object.request = request
        view_object.args = args
        view_object.kwargs = kwargs
        return view_object

    def view(self, request, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        self._load_middleware()
        if self.middleware:
            response = self._middleware_chain(request)
        else:
            response = None
            for middleware_method in self._request_middleware:
                response = middleware_method(request)
                if response:
                    break

            if response is None:
                response = self._get_response(request)

        for middleware_method in self._response_middleware:
            response = middleware_method(request, response)

        return response

    def _load_middleware(self):
        self._request_middleware = []
        self._view_middleware = []
        self._template_response_middleware = []
        self._response_middleware = []
        self._exception_middleware = []

        if self.middleware is None:
            middleware_classes = self.middleware_classes or []
            for mw_class in middleware_classes:
                if isinstance(mw_class, tuple):
                    mw_class, mw_types = mw_class[0], mw_class[1:]
                else:
                    mw_types = None
                mw_instance = mw_class()

                if self._add_middleware(mw_instance, mw_types, MiddlewareType.PROCESS_REQUEST):
                    self._request_middleware.append(mw_instance.process_request)
                if self._add_middleware(mw_instance, mw_types, MiddlewareType.PROCESS_VIEW):
                    self._view_middleware.append(mw_instance.process_view)
                if self._add_middleware(mw_instance, mw_types, MiddlewareType.PROCESS_TEMPLATE_RESPONSE):
                    self._template_response_middleware.insert(0, mw_instance.process_template_response)
                if self._add_middleware(mw_instance, mw_types, MiddlewareType.PROCESS_RESPONSE):
                    self._response_middleware.insert(0, mw_instance.process_response)
                if self._add_middleware(mw_instance, mw_types, MiddlewareType.PROCESS_EXCEPTION):
                    self._exception_middleware.insert(0, mw_instance.process_exception)
        else:
            if django.VERSION < (1, 10):
                raise NotImplementedError('New style middleware is not intended to be used with older django versions')
            from django.core.handlers.exception import convert_exception_to_response
            handler = convert_exception_to_response(self._get_response)
            middleware_classes = reversed(self.middleware or [])
            for mw_class in middleware_classes:
                if isinstance(mw_class, tuple):
                    mw_class, mw_types = mw_class[0], mw_class[1:]
                else:
                    mw_types = None
                mw_instance = mw_class(handler)

                if self._add_middleware(mw_instance, mw_types, MiddlewareType.PROCESS_VIEW):
                    self._view_middleware.insert(0, mw_instance.process_view)
                if self._add_middleware(mw_instance, mw_types, MiddlewareType.PROCESS_TEMPLATE_RESPONSE):
                    self._template_response_middleware.append(mw_instance.process_template_response)
                if self._add_middleware(mw_instance, mw_types, MiddlewareType.PROCESS_EXCEPTION):
                    self._exception_middleware.append(mw_instance.process_exception)

                handler = convert_exception_to_response(mw_instance)
            self._middleware_chain = handler

    def _get_response(self, request):
        response = None
        for middleware_method in self._view_middleware:
            response = middleware_method(request, self._run_view, self.args, self.kwargs)
            if response:
                break
        if response is None:
            try:
                response = self._run_view(request)
            except Exception as e:
                response = self._process_exception_by_middleware(e, request)
        if hasattr(response, 'render') and callable(response.render):
            for middleware_method in self._template_response_middleware:
                response = middleware_method(request, response)

        return response

    def _process_exception_by_middleware(self, exception, request):
        for middleware_method in self._exception_middleware:
            response = middleware_method(request, exception)
            if response:
                return response
        raise

    def _add_middleware(self, mw_instance, mw_types, middleware_type):
        return hasattr(mw_instance, middleware_type) and (not mw_types or middleware_type in mw_types)

    def _run_view(self, request):
        if self.view_class:
            response = self.view_class.as_view(**self.get_view_kwargs())(request, *self.args, **self.kwargs)
        elif self.view_function:
            response = self.__class__.__dict__['view_function'](request, *self.args, **self.kwargs)
        return response


class ViewTransactionTestCase(ViewTestCaseMixin, django_test.TransactionTestCase):
    pass


class ViewTestCase(ViewTestCaseMixin, django_test.TestCase):
    pass


if django.VERSION >= (1, 4):
    class ViewLiveServerTestCase(ViewTestCaseMixin, django_test.LiveServerTestCase):
        pass

if django.VERSION >= (1, 5):
    class ViewSimpleTestCase(ViewTestCaseMixin, django_test.SimpleTestCase):
        pass
